#include "EcPreCompile.h" //QApplication

#include "ServicePublisher.h"


/*!
 * \brief GameInfo::GameInfo 默认构造函数。
 */
EqNetworkThread::EqNetworkThread()
{
    initializeMemberObjects(); //初始化成员对象。
    connectSignals(); //连接信号槽。

} //ServicePublisher::ServicePublisher()

/*!
 * \brief ServicePublisher::publishService 发布服务。
 * \param LanServiceName 服务名字。
 * \param LanServicePort 服务端口。
 * \param LanServiceProtocolType 服务协议类型。
 */
void EqNetworkThread::publishService(string LanServiceName,quint16 LanServicePort,exist::ServicePublishMessage::ServiceProtocolType LanServiceProtocolType)
{
    Q_UNUSED(LanServiceName); //不使用这个参数。
    Q_UNUSED(LanServicePort); //不使用这个参数。
    Q_UNUSED(LanServiceProtocolType); //不使用这个参数。

    exist::ExistMessage existMessage; //消息。

    existMessage.set_messagetype (exist::ExistMessage::SERVICEPUBLISH); //设置消息类型。

    exist::ServicePublishMessage * servicePublishMessage=existMessage.mutable_servicepublishmessage (); //获取服务发布消息对象。

    servicePublishMessage->set_name (LanServiceName); //设置服务名字。
    servicePublishMessage->set_port (LanServicePort); //设置端口号。
    servicePublishMessage->set_protocoltype (LanServiceProtocolType); //设置协议类型。

    string packageString=existMessage.SerializeAsString (); //序列化为字符串。

//    QByteArray datagram=QByteArray::fromStdString (packageString); //转换成字符数组。

    return;
} //void ServicePublisher::publishService(QString LanServiceName,quint16 LanServicePort,ServiceProtocolType LanServiceProtocolType)

/*!
 * \brief ServicePublisher::connectSignals 连接信号槽。
 */
void EqNetworkThread::connectSignals()
{


    return;
} //void ServicePublisher::connectSignals()

/*!
 * \brief EqNetworkThread::run 运行。
 */
void EqNetworkThread::run()
{
//    socket.setT
    socket.setReceiveTimeout(Poco::Timespan(5,0)); //每次接收，等待5秒超时。

    while(true)
    {

        try
        {
            char buffer[512];

            Poco::Net::SocketAddress sender;
            Poco::Timestamp currentTime;


            string timestamp=Poco::DateTimeFormatter::format(currentTime,"%Y-%m-%d %H:%M:%S"); //格式化时间。

            int n=socket.receiveFrom (buffer,sizeof(buffer)-1,sender);

            Poco::Timestamp currentTimeAfter;


            string timestampAfter=Poco::DateTimeFormatter::format(currentTimeAfter,"%Y-%m-%d %H:%M:%S") ; //格式化时间。


            if (n>0) //读入了有效的数据。
            {
                readedSocketData(buffer,n,sender); //发射信号，读取到了数据。
            } //if (n>0) //读入了有效的数据。

            Poco::Thread::sleep(200); //睡眠。
        } //try
        catch (Poco::TimeoutException & e)
        {
        } //catch (Poco::TimeoutException & e)
    }

    return;
} //void EqNetworkThread::run()

/*!
 * \brief EqNetworkThread::readedSocketData 信号，读取到了数据。
 * \param buffer 数据缓冲区。
 * \param bufferLength 缓冲区长度。
 * \param sender 发送者。
 */
void EqNetworkThread::readedSocketData(char * buffer,int bufferLength,Poco::Net::SocketAddress sender)
{
    processDatagram(buffer,bufferLength,sender);

} //void EqNetworkThread::readedSocketData(char * buffer,int bufferLength,Poco::Net::SocketAddress sender)

/*!
 * \brief ServicePublisher::debugWriteDataGram 输出数据包的内容到文件。
 * \param datagram 数据包内容。
 * \param datagramLength 数据包长度。
 */
void EqNetworkThread::debugWriteDataGram(char * datagram,int datagramLength)
{
    Q_UNUSED(datagram);
    Q_UNUSED(datagramLength);


} //void ServicePublisher::debugWriteDataGram(char * datagram,int datagramLength)


/*!
 * \brief UdpServer::processDatagram 处理数据包。
 * \param datagram 数据包。
 */
void EqNetworkThread::processDatagram(char *  datagram,int datagramLength, Poco::Net::SocketAddress senderAddress)
{

    debugWriteDataGram(datagram,datagramLength); //Debug.

    Q_UNUSED(senderAddress); //不使用勒个参数。

    ExistMessage existMessage;
//    existMessage.ParseFromString (datagram.toStdString ()); //解析数据包。


    if (datagramLength>0) //长度有效。
    {
      string datagramtoStdString(datagram,datagramLength); //数据包。



      bool parseResult=existMessage.ParseFromString (datagramtoStdString); //解析数据包。

      Q_UNUSED(parseResult);



      switch (existMessage.messagetype ()) //根据消息类型来处理。
      {
      case ExistMessage::SERVICEPROBE: //服务扫描。
          replyServiceProbe(existMessage,senderAddress.host().toString()); //回复服务探测数据包。

          break; //跳出。

      default: //默认情况。
          break; //跳出。
      } //switch (existMessage.messagetype ()) //根据消息类型来处理。

    } //if (datagramLength>0) //长度有效。


    return;
} //void UdpServer::processDatagram(QByteArray datagram)

/*!
 * \brief EqNetworkThread::replyServiceProbe 回复服务探测数据包。
 * \param existMessage 消息。
 * \param senderAddresshosttoString 发送者的地址。
 */
void EqNetworkThread::replyServiceProbe(ExistMessage existMessage,string senderAddresshosttoString)
{
    const ServiceProbeMessage  servicePublishMessage=existMessage.serviceprobemessage (); //获取消息发布消息对象。

    quint16 servicePort=servicePublishMessage.port (); //获取端口号。

    //得到了IP和端口，可以回复了:

    castData(cachedServiceData,senderAddresshosttoString,servicePort); //发布数据到指定IP的指定端口。


} //void EqNetworkThread::replyServiceProbe(ExistMessage existMessage,string senderAddresshosttoString)


/*!
 * \brief UdpServer::readSocketData 数据来了，则读取。
 */
void EqNetworkThread::readSocketData()
{
    try
    {
        char buffer[512];

        Poco::Net::SocketAddress sender;

        int n=socket.receiveFrom (buffer,sizeof(buffer)-1,sender);


        if (n>0) //读入了有效的数据。
        {
            std::cout << "after read,read bytes:" << n << std::endl; //Debugt.

    //        buffer[n]='\0';

            std::cout << sender.toString () << ": " << buffer << std::endl;

        } //if (n>0) //读入了有效的数据。

    } //try
    catch (Poco::TimeoutException & e)
    {
    } //catch (Poco::TimeoutException & e)

    return;
} //void UdpServer::readSocketData()

/*!
 * \brief SBrowserWindow::initializeMemberObjects 初始化成员对象。
 */
void EqNetworkThread::initializeMemberObjects()
{
    //加入组播组：
    Poco::Net::SocketAddress address("239.173.40.5",MulticastPort);
        Poco::Net::SocketAddress localAddr(Poco::Net::IPAddress(),address.port()); //本地地址。

    socket=Poco::Net::MulticastSocket(localAddr,true);
    socket.joinGroup (address.host ());
    socket.setBlocking (false); //非阻塞。

    //定时器：

    return;
} //void SBrowserWindow::initializeMemberObjects()

/*!
 * \brief EqNetworkThread::castData 发布数据到指定IP的指定端口。
 * \param cachedServiceDataToSend 要发送的数据。
 * \param senderAddresshosttoString 指定的IP。
 * \param servicePort 指定的端口。
 */
void EqNetworkThread::castData(string cachedServiceDataToSend,string senderAddresshosttoString,quint16 servicePort)
{

    try //尝试发送数据。
    {
        std::string msg=cachedServiceDataToSend; //消息内容。

        Poco::Net::SocketAddress sa(senderAddresshosttoString,servicePort);
        Poco::Net::DatagramSocket dgs;
        dgs.connect (sa);

        dgs.sendBytes (msg.data (),msg.size ()); //发送数据包。
    } //try //尝试发送数据。
    catch (Poco::Net::NetException & e)
    {
        std::cout << __FILE__ << __LINE__ << __func__ << "Net exception:" << e.what (); //报告错误。
    } //catch (Poco::Net::NetException & e)

} //void EqNetworkThread::castData(string cachedServiceDataToSend,string senderAddresshosttoString,quint16 servicePort)

/*!
 * \brief EqNetworkThread::castData 发送数据。
 * \param dataToCast 要发送的数据。
 */
void EqNetworkThread::castData(string dataToCast)
{
    cachedServiceData=dataToCast; //记录数据。

    castData(dataToCast,"239.173.40.5",MulticastPort);

    return;
} //void EqNetworkThread::castData(QByteArray dataToCast)
